/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under Perforce control
	$Id: //depot/express/fcs992/include/avs/array.h#1 $
*/
#if !defined(array_h)
#define array_h

#include <assert.h>


class DBArray {

public:

// Construction
	DBArray(int growBy = 0);

// Attributes
	int     size() const;
	int     upperBound() const;
	void	size(int newSize, int growBy = -1);

// Operations
	// Clean up
	void    freeExtra();
	void	removeAll();

	// Accessing elements
	void*	atGet(int index) const;
	void	atSet(int index, void* newElement);
	void*&	elementAt(int index);
	int	find(void * element);

	// Potentially growing the array
	void	atSetGrow(int index, void* newElement);
	int	add(void* newElement);

	// Stack-like operations
	void 	push (void* newElement);
	void*	pop ();

	// overloaded operator helpers
	void*	operator[](int index) const;
	void*&	operator[](int index);

	// Operations that move elements around
	void	insertAt(int index, void* newElement, int count = 1);
	void	insertAt(int startIndex, DBArray* newArray);
	void	removeAt(int index, int count);
	void *	removeAt(int index);
	void *	remove(void *element);

// Implementation
protected:
	void**	 _data;       // the actual array of data
	int	_size;	      // # of elements (upperBound - 1)
	int	_maxSize;     // max allocated
	int	_growBy;      // grow amount

public:
	~DBArray();
};

// Inline Functions

inline	int
DBArray::size() const { return _size; }

inline	int     
DBArray::upperBound() const { return _size-1; }

inline void	
DBArray::removeAll() { DBArray::size(0); }

	// Accessing elements
inline void * 
DBArray::atGet(int index) const
{
	assert(index >= 0 && index < _size);
	return _data[index];
}

inline void 
DBArray::atSet(int index, void* newElement)
{
	assert(index >= 0 && index < _size);
	_data[index] = newElement;
}

inline void*&
DBArray::elementAt(int index)
{
	assert(index >= 0 && index < _size);
	return _data[index];
}

inline int
DBArray::add(void* newElement)
{
	int index = _size;
	atSetGrow(index, newElement);
	return index;
}

	// Stack-like operations
inline void	
DBArray::push (void* newElement)
{
    add(newElement);
}

inline void*
DBArray::pop ()
{
    return removeAt(_size - 1);
}

        // overloaded operator helpers

inline void*
DBArray::operator[](int index) const { return atGet(index); }

inline void*&
DBArray::operator[](int index) { return elementAt(index); }

#endif
/////////////////////////////////// EOF //////////////////////////////////////
